import{_ as a,l as s,I as n,e as i}from"./index-BUHGIoA-.js";import{p}from"./treemap-KZPCXAKY-C_SScpQ8.js";import"./_baseUniq-DKknK1CS.js";import"./_basePickBy-BeyvkOJl.js";import"./clone-BFehDndA.js";var g={parse:a(async r=>{const e=await p("info",r);s.debug(e)},"parse")},v={version:"11.13.0"},d=a(()=>v.version,"getVersion"),m={getVersion:d},c=a((r,e,o)=>{s.debug(`rendering info diagram
`+r);const t=n(e);i(t,100,400,!0),t.append("g").append("text").attr("x",100).attr("y",40).attr("class","version").attr("font-size",32).style("text-anchor","middle").text(`v${o}`)},"draw"),l={draw:c},y={parser:g,db:m,renderer:l};export{y as diagram};
