import{s as a,c as s,a as e,C as t}from"./chunk-WL4C6EOR-EuLKGdLN.js";import{_ as i}from"./index-BUHGIoA-.js";import"./chunk-FMBD7UC4-BV0qEgpm.js";import"./chunk-JSJVCQXG-BkS2FUUZ.js";import"./chunk-55IACEB6-BSLrC8xV.js";import"./chunk-KX2RTZJC-y3tzirwE.js";var u={parser:e,get db(){return new t},renderer:s,styles:a,init:i(r=>{r.class||(r.class={}),r.class.arrowMarkerAbsolute=r.arrowMarkerAbsolute},"init")};export{u as diagram};
