import{e as O,c as p,g as m,k as P,h as c,j as l,l as w,m as A,n as I,t as N,o as _}from"./_baseUniq-DKknK1CS.js";import{aW as g,aC as $,aX as y,aY as F,aZ as M,a_ as x,a$ as B,b0 as E,b1 as T}from"./index-BUHGIoA-.js";var S=/\s/;function C(r){for(var n=r.length;n--&&S.test(r.charAt(n)););return n}var G=/^\s+/;function H(r){return r&&r.slice(0,C(r)+1).replace(G,"")}var o=NaN,R=/^[-+]0x[0-9a-f]+$/i,W=/^0b[01]+$/i,X=/^0o[0-7]+$/i,Y=parseInt;function q(r){if(typeof r=="number")return r;if(O(r))return o;if(g(r)){var n=typeof r.valueOf=="function"?r.valueOf():r;r=g(n)?n+"":n}if(typeof r!="string")return r===0?r:+r;r=H(r);var t=W.test(r);return t||X.test(r)?Y(r.slice(2),t?2:8):R.test(r)?o:+r}var v=1/0,z=17976931348623157e292;function K(r){if(!r)return r===0?r:0;if(r=q(r),r===v||r===-v){var n=r<0?-1:1;return n*z}return r===r?r:0}function L(r){var n=K(r),t=n%1;return n===n?t?n-t:n:0}function ar(r){var n=r==null?0:r.length;return n?p(r):[]}var b=Object.prototype,Z=b.hasOwnProperty,er=$(function(r,n){r=Object(r);var t=-1,e=n.length,a=e>2?n[2]:void 0;for(a&&y(n[0],n[1],a)&&(e=1);++t<e;)for(var f=n[t],i=F(f),s=-1,d=i.length;++s<d;){var u=i[s],h=r[u];(h===void 0||M(h,b[u])&&!Z.call(r,u))&&(r[u]=f[u])}return r});function ir(r){var n=r==null?0:r.length;return n?r[n-1]:void 0}function D(r){return function(n,t,e){var a=Object(n);if(!x(n)){var f=m(t);n=P(n),t=function(s){return f(a[s],s,a)}}var i=r(n,t,e);return i>-1?a[f?n[i]:i]:void 0}}var J=Math.max;function Q(r,n,t){var e=r==null?0:r.length;if(!e)return-1;var a=t==null?0:L(t);return a<0&&(a=J(e+a,0)),c(r,m(n),a)}var sr=D(Q);function U(r,n){var t=-1,e=x(r)?Array(r.length):[];return l(r,function(a,f,i){e[++t]=n(a,f,i)}),e}function fr(r,n){var t=B(r)?w:U;return t(r,m(n))}var V=Object.prototype,k=V.hasOwnProperty;function j(r,n){return r!=null&&k.call(r,n)}function ur(r,n){return r!=null&&A(r,n,j)}function rr(r,n,t,e){if(!g(r))return r;n=I(n,r);for(var a=-1,f=n.length,i=f-1,s=r;s!=null&&++a<f;){var d=N(n[a]),u=t;if(d==="__proto__"||d==="constructor"||d==="prototype")return r;if(a!=i){var h=s[d];u=void 0,u===void 0&&(u=g(h)?h:E(n[a+1])?[]:{})}T(s,d,u),s=s[d]}return r}function dr(r,n,t){for(var e=-1,a=n.length,f={};++e<a;){var i=n[e],s=_(r,i);t(s,i)&&rr(f,I(i,r),s)}return f}export{dr as a,U as b,sr as c,er as d,L as e,ar as f,ur as h,ir as l,fr as m,K as t};
